package edu.chl.hajo.monopoly.core;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import edu.chl.hajo.monopoly.constants.Constants;
import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.util.Dump;
import edu.chl.hajo.monopoly.util.IObservable;

/**
 * Class representing the over all game
 * @author hajo
 *
 */
public class Game implements IObservable {
	
	private final PropertyChangeSupport observers = new PropertyChangeSupport(
			this);
	private final Dice[] dices = new Dice[] { new Dice(), new Dice() };
	private final List<Player> players = new ArrayList<Player>();
	// private final Board board = new Board();
	private Player actualPlayer;

	private static Game instance;

	// TODO Make it a singleton (for now?)
	public static Game getInstance() {
		if (instance == null) {
			instance = new Game();
		}
		return instance;
	}

	private Game() {
	}

	public Player getActualPlayer() {
		return actualPlayer;
	}

	public void startGame() {
		actualPlayer = players.get(0);
	}

	public void addPlayer(Player player) {
		players.add(player);
		for (Player p : players) {
			Dump.dump(p.getPiece().toString());
		}
	}

	public List<Player> getPlayers() {
		List<Player> observables = new ArrayList<Player>();
		for (Player p : players) {
			observables.add(p);
		}
		return observables;
	}

	public void nextPlayer() {
		int i = (players.indexOf(actualPlayer) + 1) % players.size();
		actualPlayer = players.get(i);
		observers.firePropertyChange(StateChanges.PLAYER_NEXT.toString(), null,
				null);

	}

	public boolean moveActualPlayer() {
		int fst = dices[0].roll();
		int sec = dices[1].roll();
		Dump.dump(fst + ":" + sec);
		int steps = fst + sec;
		int position = (steps + actualPlayer.getPosition())
				% Constants.MAX_SPACES - 1;
		actualPlayer.setPosition(position);
		// TODO
		/*
		 * if( fst == sec fst == 6 && sec == 6){ return false; }else{ return
		 * true; }
		 */
		return true;
	}

	@Override
	public void addObserver(PropertyChangeListener observer) {
		observers.addPropertyChangeListener(observer);
	}

}
