package edu.chl.hajo.monopoly.view;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import edu.chl.hajo.monopoly.constants.Constants;
import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.Game;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.ctrl.EndTurnCtrl;
import edu.chl.hajo.monopoly.ctrl.MoveCtrl;
import edu.chl.hajo.monopoly.util.Dump;


@SuppressWarnings("serial")
public class MainFrame extends JFrame implements PropertyChangeListener {
	private JPanel jPnlWest;
	private JPanel jPnlNorth;
	private JLabel jLblPlayers;
	private JLabel jLblActualPlayer;
	private JButton jBtnNext;
	private JButton jBtnThrowDice;
	private JPanel jPnlSouth;
	private JPanel jPnlCenter;
	private JPanel jPnlEast;

	private List<SpacePanel> spacePanels = new ArrayList<SpacePanel>();

	public MainFrame() {
		initGUI();
		initSpacePanels();
		initCtrls();
		Game.getInstance().addObserver(this);
		List<Player> ps = Game.getInstance().getPlayers();
		for (Player p : ps) {
			p.addObserver(this);
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String name = evt.getPropertyName();
		
		if (StateChanges.PLAYER_POSITION.toString().equals(name)) {
			Integer oldPos =  (Integer) evt.getOldValue();
			Integer newPos =  (Integer) evt.getNewValue();
			
			// TODO Ugly bypassing the control
			Player p = Game.getInstance().getActualPlayer();
			Dump.dump("Player " + p.getPiece().toString() + " pos " + p.getPosition());
			
			spacePanels.get(oldPos).erasePlayer(p.getPiece());
			spacePanels.get(newPos).drawPlayer(p.getPiece());
		}else if (StateChanges.PLAYER_NEXT.toString().equals(name)) {
			// TODO Ugly bypassing the control
			//Player p = Game.getInstance().getActualPlayer();
			//Dump.dump(p.getPiece().toString());
		}

	}

	private void initCtrls() {
		jBtnThrowDice.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				new MoveCtrl().execute();
			}

		});
		jBtnNext.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				new EndTurnCtrl().execute();
			}

		});
	}

	// --------- GUI ---------------

	private void initSpacePanels() {
		for ( int i = 0; i < Constants.MAX_SPACES ; i++) {
			spacePanels.add(new SpacePanel(String.valueOf(i)));
		}
		// TODO Hard coded
		for (int i = 0; i < 11; i++) {
			jPnlNorth.add(spacePanels.get(i));
		}
		for (int i = 11; i < 20; i++) {
			jPnlEast.add(spacePanels.get(i));
		}
		for (int i = 30; i > 19; i--) {
			jPnlSouth.add(spacePanels.get(i));
		}
		for (int i = 39; i > 30; i--) {
			jPnlWest.add(spacePanels.get(i));
		}
	}

	private void initGUI() {
		this.setSize(781, 544);
		this.setTitle("Monopoly");
		BorderLayout thisLayout = new BorderLayout();
		getContentPane().setLayout(thisLayout);
		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		{
			jPnlWest = new JPanel();
			BoxLayout jPnlWestLayout = new BoxLayout(jPnlWest,
					javax.swing.BoxLayout.Y_AXIS);
			jPnlWest.setLayout(jPnlWestLayout);
			getContentPane().add(jPnlWest, BorderLayout.WEST);
		}
		{
			jPnlNorth = new JPanel();
			BoxLayout jPnlNorthLayout = new BoxLayout(jPnlNorth,
					javax.swing.BoxLayout.X_AXIS);
			jPnlNorth.setLayout(jPnlNorthLayout);
			getContentPane().add(jPnlNorth, BorderLayout.NORTH);
		}
		{
			jPnlEast = new JPanel();
			BoxLayout jPnlEastLayout = new BoxLayout(jPnlEast,
					javax.swing.BoxLayout.Y_AXIS);
			jPnlEast.setLayout(jPnlEastLayout);
			getContentPane().add(jPnlEast, BorderLayout.EAST);
		}
		{
			jPnlCenter = new JPanel();
			getContentPane().add(jPnlCenter, BorderLayout.CENTER);
			{
				jLblPlayers = new JLabel();
				jPnlCenter.add(jLblPlayers);
				jLblPlayers.setText("jLblPlayers");
				jLblPlayers.setBorder(BorderFactory
						.createTitledBorder("Players"));
			}
			{
				jBtnThrowDice = new JButton();
				jPnlCenter.add(jBtnThrowDice);
				jBtnThrowDice.setLayout(null);
				jBtnThrowDice.setText("jBtnThrowDice");
			}
			{
				jBtnNext = new JButton();
				jPnlCenter.add(jBtnNext);
				jBtnNext.setText("jBtnNext");
			}
			{
				jLblActualPlayer = new JLabel();
				jPnlCenter.add(jLblActualPlayer);
				jLblActualPlayer.setText("jLblActualPlayer");
				jLblActualPlayer.setBorder(BorderFactory
						.createTitledBorder("ActualPlayer"));
			}
		}
		{
			jPnlSouth = new JPanel();
			BoxLayout jPnlSouthLayout = new BoxLayout(jPnlSouth,
					javax.swing.BoxLayout.X_AXIS);
			jPnlSouth.setLayout(jPnlSouthLayout);
			getContentPane().add(jPnlSouth, BorderLayout.SOUTH);
		}

	}

}
