package edu.chl.hajo.monopoly;

import java.util.ArrayList;
import java.util.List;

import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.Game;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.view.MainFrame;

/**
 * ITERATION TWO 
 * 
 * This is foremost a redesign of iteration 1.0. After this iteration we should
 * have a much stabler design. Have to work on until convinced!! Trying to
 * cover tricky parts.
 * 
 * Here we also should considering if (which) framework to use. Is there
 * really any benefits? Do we see cases where it would simplify?
 * 
 * NEW USE CASES 
 * - Buy property 
 * - Sell property
 * 
 * Both tricky, but do it as simple as possible (not checking everything)
 * 
 * A lot here has to do with the HCI (see GUI from RAD)
 * - To buy/sell a property there should be a pop up dialog (see package
 *   details) with buttons Buy and Sell. The buttons should be
 *   enabled/disabled correctly (i.e no owner Buy possible etc)
 * - Enable/disable is done from actionListeners or propertyChanges method
 * 	 both in the actual GUI frame or panel. 
 * 
 * 
 * REDESIGN 
 * - Move functionality from core to controls. Many things have to be
 *   checked in the controls, is player in jail?, has player money to buy? are the
 *   dices equal? who is the owner? etc. Would be to messy to have in model.
 *   Now there is a 1:1 match between controls and use cases
 * - Spaces hierarchy has been added. Common code in AbstractSpace 
 * - The views (SpacePanels) of the Spaces is also a hierarchy (a 1:1 one match). Each Panel
 *   has a reference to the Space in the model. Common code in AbstractSpacePanel
 * - Previously SpacePanels observed the actual player (to display on Panel).
 *   Now a Panel only displays a model Space (the model Space is holding all visiting
 *   Player, will display all)
 * - Player will hold a list of properties (not really necessary but simple if we would
 *   like to display all properties owned by this player.  
 * 
 * NEW DESIGN 
 * - A simple unified interface to Game (IGame) clarifying the functionality 
 * - A file (data) format for the spaces, see resources/spacesData.txt
 *   With some work i should be possible to configure the spaces rather freely 
 * - A GameEvent system is added (using same technique as StateChanges). Previously
 *   only state changes in model was fired. Now GameEvents also are (for example
 *   both dices equal = roll again). We try to send primitive data to GUI (minimize
 *   dependencies on model)
 * - Enable/disable for roll dice and end-round buttons 
 * 
 * ADDDED SUBSYSTEMS
 * - A simple file reader sub system
 * 
 * DEPENDENCIES AND CODE QUALITY
 * - Checked with JDepand and FindBugs, no problems
 * 
 * TODOS
 * - How are we going to switch rules? The rules are hard coded ?!
 * - Add a text field to GUI to make it possible to move player a fixed nr of
 *   steps (as in the tests)
 * 
 * @author hajo
 * 
 */
public class Main {

	public static void main(String[] args) {
		List<Player> p = new ArrayList<Player>();
		// Ordering matters! Use buy game
		p.add(new Player("Olle", Piece.DOG));
		p.add(new Player("Fia", Piece.TOP_HAT));
		p.add(new Player("Pelle", Piece.CAR));
		p.add(new Player("Sanna", Piece.IRON));

		// TODO for now
		((Game) Game.getInstance()).init(p);
		new MainFrame().setVisible(true);
	}
}
