package edu.chl.hajo.monopoly.core.space;

import edu.chl.hajo.monopoly.core.IDetailsVisitor;
import edu.chl.hajo.monopoly.core.IDialogVisitor;
import edu.chl.hajo.monopoly.core.IPanelVisitor;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.event.GameEvents;
import edu.chl.hajo.monopoly.event.StateChanges;
import edu.chl.hajo.monopoly.util.Config;

//import edu.chl.hajo.monopoly.util.IVisitor;

/**
 * A space that can be owned (has owner)
 * but not have buildings
 * 
 * @author hajo
 * 
 */
public class Property extends AbstractSpace {

	protected int rent = Config.getDefaultRent();
	protected Player owner = null;
	protected int price = 500;

	public Property(String name) {
		super(name);
	}

	public void setOwner(Player owner) {
		this.owner = owner;
		if (owner == null) {
			eb.send(this, StateChanges.PROPERTY_LOST_OWNER, null, null);
		} else {
			eb.send(this, StateChanges.PROPERTY_GOT_OWNER, null, owner);
		}
	}

	public Player getOwner() {
		return owner;
	}

	public boolean hasOwner() {
		return owner != null;
	}
	
	public void setRent(int rent) {
		this.rent = rent;
	}

	public int getRent() {
		return rent;
	}
	
	public int getPrice() {
		return price;
	}
	
	public void setPrice(int price) {
		this.price = price;
	}
	
	@Override
	public void checkEvent(Player player) {
		if( owner != null && player != owner ){
			eb.broadcast(GameEvents.MUST_PAY_RENT, null, this);
		}
	}

	@Override
	public void visit(IPanelVisitor pf) {
		pf.setPanel(this);
	}
	@Override
	public void visit(IDialogVisitor df) {
		df.setDialog(this);
		
	}

	public void visit( IDetailsVisitor df ){
		df.setDetails(this);
	}
}
