package edu.chl.hajo.monopoly.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.event.StateChanges;
import edu.chl.hajo.monopoly.util.Options;

/**
 * Base class for all Space panels (views of the Spaces)
 * 
 * NOTE: This is static GUI parts, all connections to the
 * EventBus are done in GUIBuilder.
 */
@SuppressWarnings("serial")
public abstract class AbstractSpacePanel extends JPanel implements
		PropertyChangeListener {

	protected JLabel jLlbName;
	private Set<Piece> visitingPlayers = new HashSet<Piece>();
	protected String name;

	// To be overridden by subclasses
	protected abstract void showDetails();

	protected EventBus eb = EventBus.getInstance();

	protected final MouseAdapter listener = new MouseAdapter() {

		@Override
		public void mouseClicked(MouseEvent e) {
			showDetails();
		}
	};

	protected AbstractSpace space;

	public AbstractSpacePanel(AbstractSpace space) {
		this.addMouseListener(listener);
		this.space = space;
		initGUI();
		// TODO look up a real name for the space
		jLlbName.setText(space.getName());
		this.name = space.getName();
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String name = evt.getPropertyName();
		if (StateChanges.SPACE_ADD_VISITING.toString().equals(name)) {
			visitingPlayers.add((Piece) evt.getNewValue());
			this.repaint();
		} else if (StateChanges.SPACE_REMOVE_VISITING.toString().equals(name)) {
			visitingPlayers.remove((Piece) evt.getNewValue());
			this.repaint();
		}
	}

	private void initGUI() {
		try {
			{
				this.setPreferredSize(new Dimension(80, 80));
				this.setBorder(BorderFactory
						.createBevelBorder(BevelBorder.LOWERED));
				{
					jLlbName = new JLabel();
					this.add(jLlbName);
					jLlbName.setText("JLblName");
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// All spaces need to show players on space
	public void paintComponent(Graphics g) {

		super.paintComponent(g);
		int pos = 10;
		for (Piece p : visitingPlayers) {
			Color c = g.getColor();
			g.setColor(Color.BLACK);
			g.fillRect(pos, 30, 10 + 3, 10 + 3);
			g.setColor(Options.getColorForPiece(p));
			g.fillRect(pos, 30, 10, 10);
			g.setColor(c);
			pos += 10;
		}
	}
}
