package edu.chl.hajo.monopoly.ctrl;

import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.core.space.Tax;

/**
 * A control factory. To be able to switch the controls we could use
 * a factory (possible must do if switching rules)
 *
 * NOTE: This could be replace by dependency injection
 * 
 * @author hajo
 *
 */
public class CF {

	// One way to do it ------------------
	public enum Name {
		BUY_BUILDING,
		BUY_PROPERTY,
		CHANCE_COMMUNITY,
		END_TURN,
		FORCE_WITHDRAW,
		FREE_FROM_JAIL,
		GET_FROM_ALL,
		GO_JAIL,
		KILL_PLAYER,
		MOVE,
		NEW_GAME,
		PAY_RENT,
		PAY_TAX,
		SELL_BUILDING,
		SELL_PROPERTY
	}

	// Possible to return different controls depending on rules
	// Possible to cache controls
	public static IControl get(Name name, Object... args) {
		IControl ctrl = null;
		switch (name) {
		case BUY_BUILDING:
			ctrl = new BuyBuildingCtrl((Street) args[0]);
			break;
		case BUY_PROPERTY:
			ctrl = new BuyPropertyCtrl((Property) args[0]);
			break;
		case CHANCE_COMMUNITY:
			ctrl = new ChanceCommunityCtrl((String) args[0]);
			break;
		case END_TURN:
			ctrl = new EndTurnCtrl();
			break;
		case FORCE_WITHDRAW:
			ctrl = new ForceWithDrawCtrl((Integer) args[0]);
			break;
		case FREE_FROM_JAIL:
			ctrl = new FreeFromJailCtrl();
			break;
		case GET_FROM_ALL:
			ctrl = new GetFromAllCtrl((Integer) args[0]);
			break;
		case GO_JAIL:
			ctrl = new GoJailCtrl();
			break;
		case KILL_PLAYER:
			ctrl = new KillPlayerCtrl();
			break;
		case MOVE:
			if (args.length > 0) {
				ctrl = new MoveCtrl((Integer) args[0]);
			} else {
				ctrl = new MoveCtrl();
			}
			break;
		case NEW_GAME:
			ctrl = new NewGameCtrl();
			break;
		case PAY_RENT:
			ctrl = new PayRentCtrl((Property) args[0]);
			break;
		case PAY_TAX:
			ctrl = new PayTaxCtrl((Tax) args[0]);
			break;
		case SELL_BUILDING:
			ctrl = new SellBuildingCtrl((Street) args[0]);
			break;
		case SELL_PROPERTY:
			ctrl = new SellPropertyCtrl((Property) args[0]);
			break;
		default:
			throw new IllegalArgumentException("No such control");
		}
		return ctrl;
	}

}
