package edu.chl.hajo.monopoly.rules;

import edu.chl.hajo.monopoly.core.Locator;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Street;

/**
 * Helper class to check rules for buy and sell and more
 * Used by model and GUI
 * @author hajo
 *
 */
public class Rules {
	
	private static Player getPlayer(){
		return Locator.getGame().getActualPlayer();
	}
	
	// The owner is actual player
	public static boolean sellPropertyEnabled(Property property){
		return property.hasOwner() && property.getOwner() == getPlayer();
	}
	
	// No one owns and player is visiting the space
	public static boolean buyEnabled(Property property){
		return !property.hasOwner() && property.isVisitingPlayer(getPlayer());
	}
	
	// The owner is actual player and no buildings
	public static boolean sellStreetEnabled(Street street){
		return sellPropertyEnabled(street) && street.getNrOfBuildings() == 0;
	}
	
	public static boolean buyBuildingEnabled(Street street){
		return sellPropertyEnabled(street) && street.getNrOfBuildings() < 5;
	}
	
	public static boolean sellBuildingEnabled(Street street){
		return sellPropertyEnabled(street) && street.getNrOfBuildings() > 0;
	}
	
	public static boolean playerCanPay( int amount ){
		return getPlayer().getBalance() >= amount;
	}
	
	public static boolean playerBankruptcy( int amount ){
		Player p = getPlayer();
		return p.getBalance() < amount && p.getProperties().size() == 0; 
	}

	public static boolean actualPlayerInJail() {
		return getPlayer().isInJail();
	}
	
	
}
