package edu.chl.hajo.monopoly.view.details;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.WindowConstants;

import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.ctrl.CF;
import edu.chl.hajo.monopoly.rules.Rules;
import edu.chl.hajo.monopoly.view.PropertyPanel;
/**
 * Detailed view of Property (can buy/sell)
 * @author hajo
 *
 */
@SuppressWarnings("serial")
public class PropertyDetails extends AbstractSpaceDetails implements
		PropertyChangeListener {

	private JButton jBtnSell;
	private JButton jBtnBuy;

	private Property property;

	public PropertyDetails(PropertyPanel parent, Property property) {
		super(parent, property);
		this.property = property;
		eb.connect(this, property);
		initGUI();
		initControls();
	}

	@Override
	protected void onOpen() {
		setButtonsEnabled();
	}

	private void setButtonsEnabled() {
		jBtnBuy.setEnabled(Rules.buyEnabled(property));
		jBtnSell.setEnabled(Rules.sellPropertyEnabled(property));
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String e = evt.getPropertyName();

		boolean b = e.equals(StateChanges.PROPERTY_GOT_OWNER.toString())
				|| e.equals(StateChanges.PROPERTY_LOST_OWNER.toString());
		if (b) {
			setButtonsEnabled();
		}
	}

	private void initControls() {
		jBtnBuy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.BUY_PROPERTY, property).execute();
			}
		});
		jBtnSell.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.SELL_PROPERTY, property).execute();
			}
		});
	}

	private void initGUI() {
		try {
			{
				FlowLayout thisLayout = new FlowLayout();
				getContentPane().setLayout(thisLayout);
				this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
				{
					jBtnBuy = new JButton();
					getContentPane().add(jBtnBuy);
					jBtnBuy.setText("JBtnBuy");
				}
				{
					jBtnSell = new JButton();
					getContentPane().add(jBtnSell);
					jBtnSell.setText("jBtnSell");
					jBtnSell.setEnabled(false);
				}
			}
			{
				this.setSize(287, 261);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
