package edu.chl.hajo.monopoly.view.dialog;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import edu.chl.hajo.monopoly.core.Locator;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.ctrl.CF;
import edu.chl.hajo.monopoly.rules.Rules;
import edu.chl.hajo.monopoly.view.details.DetailsVistor;

/**
 * Dialog forcing player to sell to to get money to pay
 * something
 * @author hajo
 *
 */
@SuppressWarnings("serial")
public class MustSellDialog extends JDialog {
	private JButton jBtnOk;
	private JPanel jPnlProperty;
	private JPanel jPnlControl;
	private Player actualPlayer;
	private int amount; // Amount to pay

	public MustSellDialog(JFrame parent, int amount) {
		this.amount = amount;
		this.setLocationRelativeTo(parent);
		//this.actualPlayer = Game.getInstance().getActualPlayer();
		this.actualPlayer = Locator.getGame().getActualPlayer();
		initGUI();
		initCtrl();
		initPropertyPanel();
	}

	private class Listener implements ActionListener {

		private Property p;

		public Listener(Property p) {
			this.p = p;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			DetailsVistor df = new DetailsVistor();
			p.visit(df);
			df.getDetails().setVisible(true);

			// Note: property from outer class
			jBtnOk.setEnabled(Rules.playerCanPay(amount));

			if (Rules.playerBankruptcy(amount)) {
				bankrupcy();
			}
		}
	};
	
	private void initCtrl() {
		jBtnOk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				// Will success, player has sold!
				actualPlayer.withDraw(amount);
				MustSellDialog.this.dispose();
			}
		});
		this.addWindowListener(new WindowAdapter() {

			@Override
			public void windowOpened(WindowEvent e) {
				if (Rules.playerBankruptcy(amount)) {
					bankrupcy();
				}
			}
		});
	}
	
	private void initPropertyPanel() {
		List<Property> props = actualPlayer.getProperties();
		for (Property p : props) {
			JButton b = new JButton(p.getName());
			b.addActionListener(new Listener(p));
			jPnlProperty.add(b);
		}
	}
	
	private void bankrupcy(){
		JOptionPane.showMessageDialog(null, "bankrupcy",
				"Bankrupcy", JOptionPane.OK_OPTION);
		CF.get(CF.Name.KILL_PLAYER).execute();
		MustSellDialog.this.dispose();
	}

	private void initGUI() {
		this.setSize(200, 200);
		this.setTitle("Must sell or die");
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.setModalityType(ModalityType.APPLICATION_MODAL);
		this.setLayout(new BorderLayout());
		jPnlProperty = new JPanel();
		jPnlControl = new JPanel();
		jBtnOk = new JButton("jBtnOk");
		jPnlControl.add(jBtnOk);
		jBtnOk.setEnabled(false);
		this.getContentPane().add(jPnlControl, BorderLayout.SOUTH);
		this.getContentPane().add(jPnlProperty, BorderLayout.CENTER);
	}

}
