package edu.chl.hajo.mply.core;


/**
 * Piece to move around board
 * 
 * @author hajo
 * 
 */
public class Piece {

    public enum Tag {
        DOG, TOP_HAT, CAR, IRON,
    }

    private final Player owner;
    private final Tag tag; 
    private int pos = 0;
    private int oldPos;
    // Two states, possible to move or not
    private boolean movable = true;

    public Piece(Player player, Piece.Tag tag) {
        super();
        this.owner = player;
        this.tag = tag;
    }

    public void move(int steps) {
        oldPos = pos;
        pos = (pos + steps) % P.INSTANCE.getMaxSpaces();
    }
    
    // This class has the data to answer the question, let it...
    public boolean passedGo(){
        return pos < oldPos;
    }

    public int getPosition() {
        return pos;
    }

    public Player getPlayer() {
        return owner;
    }

    public Piece.Tag getTag() {
        return tag;
    }

    public void actOnCard(Card c) {
        if (c.getAction() == Card.Action.MOVE) {
            move(c.getArg());
        } else if (c.getAction() == Card.Action.ROLL_AGAIN) {
           movable = true;
        } else{
            
        }
    }

    public boolean isMovable() {
        return movable;
    }

    public void setMovable(boolean b) {
       movable = b;  
    }

    @Override
    public String toString() {
        return "PI[" + owner + "," + tag + ","+ movable + "]";
    }
    
    

}
